/*****************************************/
/* Tetris                                */
/* (C) 2000, akop karapetyan             */
/*                                       */
/* Class Game                            */
/*   This class creates necessary ins-   */
/*   tances of timer, cup and preview and*/
/*   is used as the communication medium */
/*   between classes.                    */
/*   It also maintains the score, inter- */
/*   prets events, etc...                */
/*****************************************/

import java.applet.Applet;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Panel;
import java.awt.Graphics;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.util.Vector;

/*	This class contains a single instance of a game in progress.
	Multiple Game's can be set up for multiplayer games (eventually) */
public class Game extends Panel implements KeyListener {
	
	public final int
		GAMEOVER  = 0,	/* Events, passed from Cup */
		GAMESTART = 1,
		NEXT      = 2,
		DROP	  = 3;
	
	public final int
		w = 16,
		h = 16;		/* width and height of each block, in pixels */
	
	public int 
		next,		/* Index of the next piece */
		level = 0,	/* Level decides the score and speed of game */
		score = -1, /* Score. Needs to be started at -1, as at the beginning of the game it's incremented */
		rows  = 0,	/* Number of rows dropped */
		offset = 0;	
	
	private boolean gameOver;	/* set to False when there's a game in progress */
	
	private Cup cup;
	private Preview preview;
	private Timer timer;
	
	private int myRowCounter = 0;

	/* Constructor */
	public Game(Applet p_applet, Image[] gfx) {
		
		//blockVector = blVector;
		gameOver = true;
		
		cup = new Cup(this, gfx);
		preview = new Preview(this, gfx[1], gfx[3]);
		
		addKeyListener(this);
		setBackground(p_applet.getBackground());
		
		add(cup);
		add(preview);
	}
	
	public void paint(Graphics g) {

		forceRepaint();
	}

	public void update(Graphics g) {

		paint(g);
	}

	public void forceRepaint() {
	
		cup.repaint();
		preview.repaint();
	}

	/* Returns status of the game */
	public boolean gameStatus() {
		
		return !gameOver;
	}
	
	/* Called whenever something occurs */
	public void event(int what, int extraInfo) {
		
		switch (what) {
			
			/* Next piece is coming up */
			case NEXT: {
				score += (level + 1);
				if (preview != null)
					preview.paint(preview.getGraphics());
				
				break;
			}
			/* Init vars, start the game */
			case GAMESTART: {
				gameOver = false;
				timer = new Timer(this);
				timer.pause = false;
				timer.interval = 1000;
				score = -1;
				level = 0;
				rows = 0;
				myRowCounter = 0;
				cup.newGame();
				cup.paint(cup.getGraphics());
				preview.paint(preview.getGraphics());
				startTimer();
				
				break;
			}
			/* Game is over */
			case GAMEOVER: {
				timer.pause = true;
				gameOver = true;
				cup.paint(cup.getGraphics());
				preview.paint(preview.getGraphics());
				stopTimer();
				
				break;
			}
			/* Row(s) was/were dropped */
			case DROP: {
				score += ((level + 1) * 10 * extraInfo);
				rows  += extraInfo;
				if ((myRowCounter += extraInfo) > 9) {
					level++;
					offset++;
					if (offset > 6)
						offset = 0;
					if (level < 11)
						timer.interval = 1000 - (level * 85);
					myRowCounter -= 10;
				}
				
				if (preview != null)
					preview.paint(preview.getGraphics());
				
				break;
			}
		}
	}	
	
	/* Returns a random number between 0 and n - 1 */
	private int getRandom(int n) {
		
		return (int)(Math.random() * n);
	}
	
	/* Returns the index of the next piece */
	public void getNextTrinket() {
		
		next = getRandom(7);
	}
	
	/* Called by the timer everytime a timer ticks */
	public void tick() {
		
		cup.act(2);
	}
	
	public void pauseTimer() {
		
		timer.pause = true;
	}
	
	public void resumeTimer() {
		
		timer.pause = false;
	}
	
	public void startTimer() {
		
		timer.start();
	}						
	
	public void stopTimer() {
		
		timer.stop();
	}						

	public void keyPressed(KeyEvent e) {
	
		cup.keyPressed(e);
	}

	/* Required by the interface */
	public void keyReleased(KeyEvent e) {}	
	public void keyTyped(KeyEvent e) {}	
}
