/*****************************************/
/* Tetris                                */
/* (C) 2000, akop karapetyan             */
/*                                       */
/* Class Preview                         */
/*   This class shows the score table,   */
/*   and next piece. It also re-routes   */
/*   keyboard events to the Game class   */
/*   (which then routes it to the Cup    */
/*   class).                             */
/*****************************************/

import java.awt.Graphics;
import java.awt.Canvas;
import java.awt.Image;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseListener;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

/* This class contains the preview and information window */
public class Preview extends Canvas implements 
	MouseListener, 
	KeyListener {
	
	private final int 
		w = 10,		/* Width and height of each preview piece block, in pixels */
		h = 10;
	
	private final int 
		hOffset = 15,	/* Decides how far from the edge of the canvas to draw */
		vOffset = 15;
	
	private final int 
		pw = 50,		/* Width of the piece preview window */
		ph = 50;
	
	private Game game;
	private Color background = new Color(128, 128, 128);
	
	private Image blocks;
	
	private Image buffer;	/* Used for double-buffering */
	private Graphics bufg;
	
	/* Constructor */
	public Preview(Game p_game, Image n_blocks, Image n_buf) {
		
		game = p_game;
		
		setFont(new Font("SansSerif", Font.PLAIN, 12));
		setSize(pw + (hOffset * 2) + 5, ph + 200);
		setBackground(game.getBackground());
		
		addMouseListener(this);
		addKeyListener(this);
		
		blocks = n_blocks;
		
		buffer = n_buf;
		bufg = buffer.getGraphics();
	}
	
	/* Draws the window, only called on repaint */
	private void drawWindow(Graphics g) {
		
		int bWidth = 5;
		int boxLeft = hOffset - bWidth - 10,
			boxTop = vOffset - bWidth - 10,
			boxWide = pw + (bWidth * 2) + 20,
			boxHigh = ph + (bWidth * 2) + 185;
		
		bufg.setColor(new Color(32, 32, 32));
		for (int i = 0; i < 5; i++)
			bufg.drawRect(i, i, 80, 245);
		
		g.setColor(new Color(128, 128, 128));
		for (int i = 0; i < bWidth; i++)
			g.fill3DRect(boxLeft + i, boxTop + i, 
						boxWide - i * 2, boxHigh - i * 2, true);
		
		g.draw3DRect(hOffset - 1, vOffset - 1, 
					 pw + 2, ph + 2, false);
		
		g.fill3DRect(hOffset - 1, vOffset + 61, pw + 2, 125, false);
		
		g.setColor(Color.gray);
		g.fill3DRect(hOffset, vOffset + 195, 
					 pw + 2, 22, true);
		
		g.setFont(new Font("SansSerif", Font.PLAIN, 11));
		g.setColor(Color.black);
		g.drawString("Level: ", hOffset + 7, vOffset + 87);
		g.drawString("Score: ", hOffset + 7, vOffset + 122);		
		g.drawString("Rows: ", hOffset + 7, vOffset + 157);		
		if (game.gameStatus())
			g.drawString("Quit", 33, vOffset + 211);
		else
			g.drawString("Start", 30, vOffset + 211);
		
		g.setColor(Color.white);
		g.drawString("Level: ", hOffset + 5, vOffset + 85);
		g.drawString("Score: ", hOffset + 5, vOffset + 120);		
		g.drawString("Rows: ", hOffset + 5, vOffset + 155);
		if (game.gameStatus())
			g.drawString("Quit", 31, vOffset + 210);
		else
			g.drawString("Start", 28, vOffset + 210);
	}
	
	/* Draws upcoming piece */
	public void showNextTrinket(Graphics g) {
		final int 
			x[] = {10, 5, 5, 5, 5, 5, 5},
			y[] = {10, 10, 10, 10, 5, 5, 0};
		int[][] trinket = new Trinket(game.next, 0).get2DIntArray();
		
		g.setColor(background);
		g.fillRect(hOffset, vOffset, pw, ph);

		for (int i = 0; i < trinket.length; i++)
			for (int j = 0; j < trinket[0].length; j++)
				if ( trinket[i][j] != 0 )
					g.drawImage(blocks,
						(j * w) + (x[game.next] + hOffset + 5),
						(i * h) + (y[game.next] + vOffset + 5), 
						(j * w) + (x[game.next] + hOffset + 5) + w,
						(i * h) + (y[game.next] + vOffset + 5) + h, 
						(trinket[i][j] - 1) * w, 0, 
						((trinket[i][j] - 1) * h) + h, 10, 
						this);
	}
	
	/* Returns a string composed of x characters */
	private String vbString(char c, int x) {
		String s = new String();
		
		for (int i = 0; i <= x; i++)
			s += c;
		
		return s;
	}
	
	/* Prints the score */
	public void showScore(Graphics g) {
		String 
			score = String.valueOf(game.score),
			level = String.valueOf(game.level),
			rows  = String.valueOf(game.rows);
		
		g.setColor(new Color(88, 88, 88));
		g.fillRect(hOffset + 5, 106, pw - 7, 10);
		g.fillRect(hOffset + 5, 141, pw - 7, 10);
		g.fillRect(hOffset + 5, 176, pw - 7, 10);
		
		g.setColor(Color.white);
		g.setFont(new Font("Monospaced", Font.PLAIN, 11));
		g.drawString(vbString(' ', 5 - level.length()) + level, hOffset + 5, 115);
		g.drawString(vbString(' ', 5 - score.length()) + score, hOffset + 5, 150);
		g.drawString(vbString(' ', 5 - rows.length()) + rows, hOffset + 5, 185);
	}
	
	public void update(Graphics g) {

		paint(g);
	}

	/* Called by the applet whenever a repaint is needed */
	public void paint(Graphics g) {
		
		drawWindow(bufg);
		if (game.gameStatus())
			showNextTrinket(bufg);
		showScore(bufg);
		
		g.drawImage(buffer, 0, 0, this);
	}

	public void mouseClicked(MouseEvent e){

		if ( ((e.getX() > hOffset) && (e.getX() < pw + 17)) && 
			 ((e.getY() > vOffset + 195) && (e.getY() < vOffset + 217)) ) {
			
			if (game.gameStatus())
				game.event(game.GAMEOVER, 0);
			else
				game.event(game.GAMESTART, 0);
		}
	}
	
	public void keyPressed(KeyEvent e) {
		game.keyPressed(e);
	}
	
	/* Required by the implemented classes */
	public void keyReleased(KeyEvent e) {}	
	public void keyTyped(KeyEvent e) {}
	
	public void mouseReleased(MouseEvent e){}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mousePressed(MouseEvent e){}	
}
