/*****************************************/
/* Tetris                                */
/* (C) 2000, akop karapetyan             */
/*                                       */
/* Class Trinket                         */
/*   This class creates and maintains    */
/*   trinkets - the little gamepieces.   */
/*   It mostly just contains some basic  */
/*   matrix manipulations.               */
/*****************************************/

public class Trinket {
	private int m, n;		/* m = # of rows; n = # of columns */
	private int matrix[];	/* Holds the matrix */
	
	/* Constructor. Creates a trinket of given type and angle */
	public Trinket(int gType, int gState) {
		int type2x2[] = {		/* [][] */
			1, 1,				/* [][] */
			1, 1};
		int type2x3[][] = {
			{2, 2, 0,			/* [][]   */
			 0, 2, 2},			/*   [][] */
			
			{0, 3, 3,			/*   [][] */
			 3, 3, 0}			/* [][]   */
		};
		int type3x3[][] = {
			{0, 4, 0,			/*   []   */
			 4, 4, 4,			/* [][][] */
			 0, 0, 0},
			
			{0, 0, 0,
			 5, 5, 5,			/* [][][] */
			 0, 0, 5},			/*     [] */
			
			{0, 0, 0,
			 6, 6, 6,			/* [][][] */
			 6, 0, 0}			/* []	  */
		};
		int type4x3[] = {
			0, 7, 0,			/* [][][][] */
			0, 7, 0,
			0, 7, 0,
			0, 7, 0};
		
		switch ( gType ) {
		case 0: {
					m = n = 2;
					matrix = new int[m * n];
					
					for (int i = 0; i < matrix.length; i++)
						 matrix[i] = type2x2[i];
					
					break;
				}
		case 1:
		case 2: {
					m = 2;
					n = 3;
					matrix = new int[m * n];
					
					for (int i = 0; i < matrix.length; i++)
						 matrix[i] = type2x3[gType - 1][i];
					
					break;
				}
		case 3:
		case 4:
		case 5: {
					m = n = 3;
					matrix = new int[m * n];
					
					for (int i = 0; i < matrix.length; i++)
						 matrix[i] = type3x3[gType - 3][i];
					
					break;
				}
		case 6: {
					m = 4;
					n = 3;
					matrix = new int[m * n];
					
					for (int i = 0; i < matrix.length; i++)
						 matrix[i] = type4x3[i];
					
					break;
				}
		}
		
		int state = (gState % 4);
		
		for (int i = 0; i < state; i++)
			rotateCW();
	}
	
	/* Swaps two elements in the array */
	private void swap(int a, int b) {
		int temp = matrix[a];
		
		matrix[a] = matrix[b];
		matrix[b] = temp;
	}
	
	/* Swaps two rows in the matrix */
	private void swapRows(int rA, int rB) {
		for (int i = 0; i < n; i++)
			swap( (rA * n) + i, (rB * n) + i );
	}
	
	/* Swaps two columns in the matrix */
	private void swapColumns(int cA, int cB) {
		for (int i = 0; i < m; i++)
			swap( (n * i) + cA, (n * i) + cB );
	}
	
	/* Flips the matrix vertically */
	private void vFlip() {
		for (int i = 0; i < (m / 2); i++)
			swapRows( i, m - i - 1 );
	}
	
	/* Flips the matrix horizontally */
	private void hFlip() {
		for (int i = 0; i < (n / 2); i++)
			swapColumns( i, n - i - 1 );
	}
	
	/* Transposes the matrix */
	private void transpose() {
		int newMatrix[] = new int[m * n];
		int newM = n, newN = m;
		
		for (int i = 0; i < newM; i++)
			for (int j = 0; j < newN; j++)
				newMatrix[(i * m) + j] = matrix[(j * n) + i];
		
		matrix = newMatrix;
		m = newM;
		n = newN;
	}
	
	/* Rotates the matrix clockwise */
	private void rotateCW() {
		transpose();
		hFlip();
	}
	
	/* Rotates the matrix counterclockwise */
	private void rotateCCW() {
		transpose();
		vFlip();
	}
	
	/* Returns the matrix as a two-dimensional array of integers */
	public int[][] get2DIntArray() {
		int[][] backMatrix = new int[m][n];
		
		for (int i = 0; i < (m * n); i++)
			backMatrix[i / n][i % n] = matrix[i];
		
		return backMatrix;
	}
}
